﻿MNT2 = CreateFrame('frame')
local MNT2 = MNT2
MNT2:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
MNT2:RegisterEvent("ADDON_LOADED")

local L = LibStub("AceLocale-3.0"):GetLocale("MNT2", false)

local possibleStates = { -- possible states in order of importants state 1 will always have preferance over state n
	'isMoving', -- resource intensive switch needs to be enabled to work
	'inZone', -- switch that overwrites everything below (i.e. you can assign different macroes per zone)
	'inCombat',
	'isIndoors',
	'isSwimming',
	'isFlyable',
}

local states = {} -- state of all updaters

MNT2.state = {}

setmetatable(MNT2.state, {
	__index = function(t,k)
		return states[k]
	end,
	__newindex = function (t,k,v)
		if v == states[k] then return end
		--print("* MNT2 update of element |cffffd200" .. tostring(k) .. "|r from |cffff2020" .. tostring(states[k]) .."|r to |cff20ff20" .. tostring(v).."|r")
		states[k] = v   -- update original table
		MNT2:GenerateMacro()
	end
})

MNT2.mountSpeeds = {
	[0] = L['0%'],
	[40] = L['40%'],
	[60] = L['60%'],
	[100] = L['100%'],
	[280] = L['280%'],
	[310] = L['310%'],
}

function MNT2:ADDON_LOADED(event, addon)
	if addon == "Mounted" then
		
		self:UnregisterEvent("ADDON_LOADED")
		self.ADDON_LOADED = nil
		if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
	end
end

function MNT2:PLAYER_LOGIN(event, addon)
	if GetMacroIndexByName("Mounted") == 0 then
		CreateMacro("Mounted", 1, '')
	end
	
	local defaults = {
		['char'] = {
			mounts = {
				['ground'] = {},
				['flying'] = {},
				['water'] = {},
				['aq'] = {},
			},
			mountSummonMode = {
			},
			mount_skill = 0,
		},
		['profile'] = {
			macrosets = {
				['DEFAULT'] = {
					['inCombat'] = 'Unassigned',
					['isIndoors'] = 'Unassigned',
					['isSwimming'] = 'Default Water Mount',
					['isFlyable'] = 'Default Air Mount',
					['ground'] = 'Default Ground Mount',
				},
				isMoving = {
					['inCombat'] = 'Unassigned',
					['isIndoors'] = 'Unassigned',
					['isSwimming'] = 'Unassigned',
					['isFlyable'] = 'Unassigned',
					['ground'] = 'Unassigned',
				},
			},
			activeMacroSets = {
				['DEFAULT'] = true,
				['isMoving'] = false,
			},
			instanceMacroSets = {},
		},
		['global'] = {
			['macros'] = self.defaultmacros,
		},
	}
	self.db = LibStub("AceDB-3.0"):New("MNT2DB", defaults)
	self:RegisterModuleOptions('profile', LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), L['Profile'])
	self:MountScan()
	self:SPELL_UPDATE_USABLE()
	self:ZONE_CHANGED('startup')
	self:SKILL_LINES_CHANGED()
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("SPELL_UPDATE_USABLE")
	
	-- /sigh 3 events to get zone data
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	
	-- event to track our riding skill
	self:RegisterEvent("SKILL_LINES_CHANGED")
	
	-- events to see if we are mounting up
	self:RegisterEvent('UNIT_SPELLCAST_START')
	self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED')
	
	-- we learned a new mount (lets update)
	self:RegisterEvent('COMPANION_LEARNED')
	
	if self.db.profile.activeMacroSets.isMoving then
		self:SetScript("OnUpdate", MNT2.OnUpdate)
	end
	
	SLASH_Mounted1 = "/mounted";
	SLASH_Mounted2 = "/mnt";
	SlashCmdList["Mounted"] = function()
		InterfaceOptionsFrame_OpenToCategory(self.optframe.MNT2)
	end
	
	
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

function MNT2:PLAYER_REGEN_DISABLED()
	self.state.inCombat = true
	self:SetScript("OnUpdate", nil)
	self:UnregisterEvent('UNIT_SPELLCAST_START')
	self:UnregisterEvent('UNIT_SPELLCAST_SUCCEEDED')
end

function MNT2:PLAYER_REGEN_ENABLED()
	self.state.inCombat = false
	if self.db.profile.activeMacroSets.isMoving then
		self:SetScript("OnUpdate", MNT2.OnUpdate)
	end
	self:RegisterEvent('UNIT_SPELLCAST_START')
	self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED')
end

function MNT2:SPELL_UPDATE_USABLE()
	self.state.isIndoors = IsIndoors()
	self.state.isSwimming = IsSwimming()
	self.state.isFlyable = IsFlyableArea() and IsOutdoors()
end

function MNT2:ZONE_CHANGED(event)
	self.state.inZone = GetRealZoneText()
end
MNT2.ZONE_CHANGED_INDOORS = MNT2.ZONE_CHANGED
MNT2.ZONE_CHANGED_NEW_AREA = MNT2.ZONE_CHANGED

function MNT2:SKILL_LINES_CHANGED()
	for i=1, GetNumSkillLines() do
		local name, _, _, skill = GetSkillLineInfo(i)
		if name == L['Riding'] and self.db.char.mount_skill < skill then
			self.db.char.mount_skill = skill
			print('MNT2: mount skill detected at skill level '..skill)
		end
	end
	if self.db.char.mount_skill == 75 then
		self.MaxGroundSpeed = 60
		self.MaxAirSpeed = 0
	elseif self.db.char.mount_skill == 150 then
		self.MaxGroundSpeed = 100
		self.MaxAirSpeed = 0
	elseif self.db.char.mount_skill == 225 then
		self.MaxGroundSpeed = 100
		self.MaxAirSpeed = 60
	elseif self.db.char.mount_skill == 300 then
		self.MaxGroundSpeed = 100
		if self.db.char.mounts.flying[310] then
			self.MaxAirSpeed = 310
		else
			self.MaxAirSpeed = 280
		end
	else
		self.MaxGroundSpeed = 0
		self.MaxAirSpeed = 0
	end
	self:GenerateMacro()
end

local movingState -- use to make the OnUpdate faster if we check with self.state.isMoving we do a tablelookup via a metatable function which is slower than just a local lookup
function MNT2:OnUpdate()
	if GetUnitSpeed('player') > 0 then
		if not movingState then
			movingState = true
			self.state.isMoving = true
		end
	else
		if movingState then
			movingState = false
			self.state.isMoving = false
		end
	end
end

function MNT2:UNIT_SPELLCAST_START(event, unit, spellName)
	if unit == 'player' and self.db.char.mountSummonMode[self.ReverseLocalizedIDs[spellName]] == 'cast' then
		self:GenerateMacro()
	end
end 

function MNT2:UNIT_SPELLCAST_SUCCEEDED(event, unit, spellName)
	if unit == 'player' and self.db.char.mountSummonMode[self.ReverseLocalizedIDs[spellName]] == 'instant' then
		self:GenerateMacro()
	end
end

function MNT2:COMPANION_LEARNED()
	self:MountScan() -- lets make sure one of the new mounts will be then next to summon
	self:SKILL_LINES_CHANGED() -- update our maxspeeds (incase we got a 310 mount
	self:GenerateMacro() -- update macro
end

function MNT2:MountScan()
	local newMounts = 0
	for id = 1,GetNumCompanions("MOUNT") do
		local mountSpellID = select(3, GetCompanionInfo("MOUNT", id))
		--make sure its not already found
		local mountExists
		for mountType, typeTable in pairs(self.db.char.mounts) do
			for mountSpeed, speedTable in pairs(typeTable) do
				if speedTable[mountSpellID] then
					mountExists = true
				end
			end
		end
		if not mountExists then
			newMounts = newMounts + 1
			local numVals = select('#', self:GetMountType(mountSpellID))
			local mountType, mountSpeed
			for i=1,numVals,2 do
				mountType, mountSpeed = select(i, self:GetMountType(mountSpellID))
				self.db.char.mounts[mountType][mountSpeed] = self.db.char.mounts[mountType][mountSpeed] or {}
				self.db.char.mounts[mountType][mountSpeed][mountSpellID] = 5
				self.db.char.mountSummonMode[mountSpellID] = 'cast'
			end
		end
	end
	if newMounts > 0 then
		print('MNT2: |cff20ff20'..newMounts..'|r new mounts found!')
	end
	self:LocalizeIDs()
	self:UpdateMountOptions()
	self:UpdateMainOptions()
end

function MNT2:LocalizeIDs() -- should do this everytime since we dont care what locale we are in exept for when we are casting (damn you blizzard)
	self.LocalizedIDs = {}
	self.ReverseLocalizedIDs = {}
	local TempOrderMounts = {}
	self.OrderMounts = {}
	for id in pairs(self.db.char.mountSummonMode) do
		local name = GetSpellInfo(id)
		self.LocalizedIDs[id] = name
		self.ReverseLocalizedIDs[name] = id
		TempOrderMounts[#self.OrderMounts+1] = name
	end
	table.sort(TempOrderMounts)
	for i, name in ipairs(TempOrderMounts) do
		self.OrderMounts[self.ReverseLocalizedIDs[name]] = i
	end
end

function MNT2:GenerateMacro()
	if InCombatLockdown() then return end
	local macrotable = (self.state.isMoving and self.db.profile.activeMacroSets.isMoving and self.db.profile.macrosets.isMoving) or (self.db.profile.activeMacroSets[self.state.inZone] and (not self.db.profile.instanceMacroSets[self.state.inZone] == not IsInInstance()) and self.db.profile.macrosets[self.state.inZone]) or self.db.profile.macrosets.DEFAULT
	local macroname
	for i=3,6 do
		if self.state[possibleStates[i]] and macrotable[possibleStates[i]] then
			macroname = macrotable[possibleStates[i]]
			break
		end
	end
	if not macroname then
		macroname = macrotable['ground']
	end
	local valid, macrotext = self:ParseMacro(self.db.global.macros[macroname])
	if valid then	
		self:ApplyMacro(macrotext)
	else
		self:ApplyMacro('/script print("'..tostring(macroname)..' does not seam to be a valid macro")')
	end
end

function MNT2:ApplyMacro(macrotext)
	EditMacro(GetMacroIndexByName("Mounted"), "Mounted", 1, macrotext)
end

--[==[
-- /script MNT2:StartSpeedScan()
local speed = 0-- store our speed here
local basespeed = 7 -- speed when running
local numSame =  15 -- how many consecutive frames of same speed do we need before we accept the speed
local frameCounter = 0
local flighttest = false -- ground vs flying
MNT2.SpeedScanFrame = CreateFrame('frame')
function MNT2:StartSpeedScan(fly)
	MNT2.SpeedScanFrame:SetScript("OnUpdate", MNT2.SpeedScanner)
	speed = 0
	frameCounter = 0
	if fly then
		flighttest = true
	else
		flighttest = false
	end
end

function MNT2:SpeedScanner()
	local curSpeed = GetUnitSpeed('player')
	if not InCombatLockdown() and curSpeed > 0 and IsMounted() and ((flighttest and  IsFlying()) or not flighttest)then
		if curSpeed > speed then
			speed = curSpeed
			frameCounter = 0
		elseif curSpeed < speed then
			frameCounter = 0
		elseif frameCounter == numSame then
			self:SetScript("OnUpdate", nil)
			local realspeed = math.ceil(speed/basespeed*100)-100
			local normalizedSpeed  = 0
			for speed in pairs(self.mountSpeeds) do
				if speed > normalizedSpeed and speed <= realspeed then
					normalizedSpeed = speed
				end
			end
			print('MNT2: Top Speed = '..self.mountSpeeds[normalizedSpeed])
		else
			frameCounter = frameCounter + 1
		end
	else
		frameCounter = 0
	end
end
]==]--
